@@ -14,22 +14,18 @@
 //! do so at your own risk of breakage.
 
 mod cereal;
-mod input;
-mod mime;
-mod theme;
+#[cfg(feature = "tui")]
+mod tui;
 
-pub use input::{Action, InputBinding, InputMap, KeyCombination};
-pub use theme::Theme;
+#[cfg(feature = "tui")]
+pub use tui::*;
 
-use crate::mime::MimeMap;
-use ::mime::Mime;
-use anyhow::Context;
-use editor_command::EditorBuilder;
+use editor_command::{Editor, EditorBuilder, EditorBuilderError};
 use serde::Serialize;
 use slumber_util::{
     ResultTraced, doc_link, git_link,
     paths::{self, create_parent, expand_home},
-    yaml,
+    yaml::{self, YamlError},
 };
 use std::{
     env,
@@ -37,8 +33,8 @@ use std::{
     fs::File,
     io::{self, Write},
     path::{Path, PathBuf},
-    process::Command,
 };
+use thiserror::Error;
 use tracing::{error, info};
 
 const PATH_ENV_VAR: &str = "SLUMBER_CONFIG_PATH";
@@ -48,7 +44,7 @@ const FILE: &str = "config.yml";
 /// collections. This is *not* meant to modifiable during a session. If changes
 /// are made to the config file while a TUI session is running, they won't be
 /// picked up until the app restarts.
-#[derive(Debug, Serialize)]
+#[derive(Debug, Default, Serialize)]
 #[cfg_attr(test, derive(PartialEq))]
 #[cfg_attr(feature = "schema", derive(schemars::JsonSchema))]
 #[cfg_attr(
@@ -63,49 +59,40 @@ const FILE: &str = "config.yml";
     )
 )]
 pub struct Config {
-    /// Configuration for in-app query and export commands
-    pub commands: CommandsConfig,
     /// Command to use for in-app editing. If provided, overrides
-    /// `VISUAL`/`EDITOR` environment variables. This only supports a single
-    /// command, *not* a content type map. This is because there isn't much
-    /// value in it, and plumbing the content type around to support it is
-    /// annoying.
+    /// `VISUAL`/`EDITOR` environment variables.
+    // This only supports a single command, *not* a content type map. This is
+    // because there isn't much value in it, and plumbing the content type
+    // around to support it is annoying.
+    //
+    // We could potentially convert this to `Editor` during deserialization and
+    // load env vars/defaults there. That would centralize the error handling
+    // but would mean the serialized version of the config wouldn't necessarily
+    // match what was deserialized, and could vary based on env vars.
     pub editor: Option<String>,
-    /// Command to use to browse response bodies. If provided, overrides
-    /// `PAGER` environment variable.  This could be a single command, or a map
-    /// of {content_type: command} to use different commands based on response
-    /// type. Aliased for backward compatibility with the old name.
-    #[serde(alias = "viewer", default)]
-    pub pager: MimeMap<String>,
+
+    /// HTTP engine configuration, which will be flattened for ser/de
     #[serde(flatten)]
     pub http: HttpEngineConfig,
-    /// Should templates be rendered inline in the UI, or should we show the
-    /// raw text?
-    pub preview_templates: bool,
-    /// Overrides for default key bindings
-    pub input_bindings: InputMap,
-    /// Visual configuration for the TUI (e.g. colors)
-    pub theme: Theme,
-    /// Enable debug monitor in TUI
-    ///
-    /// Mainly meant for development so don't expose it
-    #[serde(skip_serializing)]
-    #[cfg_attr(feature = "schema", schemars(skip))]
-    pub debug: bool,
-    /// Enable/disable persistence for all TUI requests? The CLI ignores this
-    /// in favor of the absence/presence of the `--persist` flag
-    pub persist: bool,
+
+    /// TUI configuration, which will be flattened for ser/de. Only included if
+    /// the `tui` flag is enabled, which allows non-TUI consumers to omit a
+    /// bunch of extra dependencies.
+    #[cfg(feature = "tui")]
+    #[serde(flatten)]
+    pub tui: tui::TuiConfig,
 }
 
 impl Config {
     /// Path to the configuration file, in this precedence:
     /// - Value of `$SLUMBER_CONFIG_PATH`
     /// - `$DATA_DIR/slumber/config.yml` **if the file exists**, where
-    ///   `$DATA_DIR` is defined by [dirs::data_dir]. This is a legacy location,
-    ///   supported for backward compatibility only. See this issue for more:
-    ///   https://github.com/LucasPickering/slumber/issues/371
+    ///   `$DATA_DIR` is defined by [paths::data_directory]. This is a legacy
+    ///   location, supported for backward compatibility only. See this issue
+    ///   for more:
+    ///   <https://github.com/LucasPickering/slumber/issues/371>
     /// - `$CONFIG_DIR/slumber/config.yml`, where `$CONFIG_DIR` is defined by
-    ///   [dirs::config_dir]
+    ///   [paths::config_directory]
     pub fn path() -> PathBuf {
         if let Ok(path) = env::var(PATH_ENV_VAR) {
             return expand_home(PathBuf::from(path)).into_owned();
@@ -123,7 +110,7 @@ impl Config {
     /// default value. This only returns an error if the file could be read, but
     /// deserialization failed. This is *not* async because it's only run during
     /// startup, when all operations are synchronous.
-    pub fn load() -> anyhow::Result<Self> {
+    pub fn load() -> Result<Self, ConfigError> {
         let path = Self::path();
         info!(?path, "Loading configuration file");
 
@@ -133,9 +120,7 @@ impl Config {
                 // Filesystem error shouldn't be fatal because it may be a
                 // weird fs error the user can't or doesn't want tofix. Just use
                 // a default config.
-                if let Some(yaml::Error::Io { error, .. }) =
-                    error.downcast_ref::<yaml::Error>()
-                {
+                if let yaml::YamlErrorKind::Io { error, .. } = &error.kind {
                     error!(
                         error = error as &dyn Error,
                         "Error opening config file {path:?}"
@@ -152,67 +137,30 @@ impl Config {
                 } else {
                     // Error occurred during deserialization - the user probably
                     // wants to fix this
-                    Err(error)
+                    Err(error.into())
                 }
             }
         }
     }
 
-    /// Get a command to open the given file in the user's configured editor.
+    /// Get an [Editor] to open the given file in the user's configured editor.
     /// Default editor is `vim`. Return an error if the command couldn't be
     /// built.
-    pub fn editor_command(&self, file: &Path) -> anyhow::Result<Command> {
+    pub fn editor(&self) -> Result<Editor, EditorError> {
         EditorBuilder::new()
             // Config field takes priority over environment variables
-            .source(self.editor.as_deref())
+            .string(self.editor.as_deref())
             .environment()
-            .source(Some("vim"))
-            .path(file)
+            .string(Some("vim"))
             .build()
-            .with_context(|| {
-                format!(
-                    "Error opening editor; see {}",
-                    doc_link("user_guide/tui/editor"),
-                )
-            })
-    }
-
-    /// Get a command to open the given file in the user's configured file
-    /// pager. Default is `less` on Unix, `more` on Windows. Return an error
-    /// if the command couldn't be built.
-    pub fn pager_command(
-        &self,
-        file: &Path,
-        mime: Option<&Mime>,
-    ) -> anyhow::Result<Command> {
-        // Use a built-in pager
-        let default = if cfg!(windows) { "more" } else { "less" };
-
-        // Select command from the config based on content type
-        let config_command = mime
-            .and_then(|mime| self.pager.get(mime))
-            .map(String::as_str);
-
-        EditorBuilder::new()
-            // Config field takes priority over environment variables
-            .source(config_command)
-            .source(env::var("PAGER").ok())
-            .source(Some(default))
-            .path(file)
-            .build()
-            .with_context(|| {
-                format!(
-                    "Error opening pager; see {}",
-                    doc_link("user_guide/tui/editor"),
-                )
-            })
+            .map_err(EditorError)
     }
 
     /// When the config file fails to open, we'll attempt to create a new one
     /// with placeholder content. Whether or not the
     // create succeeds, we're going to just log the error and use a
     // default config.
-    fn create_new(path: &Path) -> anyhow::Result<()> {
+    fn create_new(path: &Path) -> Result<(), ConfigError> {
         // You could do this read/create all in one operation using
         // OpenOptions::new().create(true).append(true).read(true),
         // but that requires write permission on the file even if it
@@ -230,7 +178,10 @@ impl Config {
                 file.write_all(&Self::default_content())?;
                 Ok(())
             })
-            .context("Error creating config file {path:?}")
+            .map_err(|error| ConfigError::Create {
+                path: path.to_owned(),
+                error,
+            })
     }
 
     /// Pre-populated content for a new config file. Include all default values
@@ -252,22 +203,6 @@ impl Config {
     }
 }
 
-impl Default for Config {
-    fn default() -> Self {
-        Self {
-            commands: CommandsConfig::default(),
-            editor: Default::default(),
-            pager: Default::default(),
-            http: Default::default(),
-            preview_templates: true,
-            input_bindings: Default::default(),
-            theme: Default::default(),
-            debug: false,
-            persist: true,
-        }
-    }
-}
-
 /// Configuration for the engine that handles HTTP requests
 #[derive(Debug, Serialize)]
 #[cfg_attr(test, derive(PartialEq))]
@@ -302,37 +237,22 @@ impl Default for HttpEngineConfig {
     }
 }
 
-/// Configuration for in-app query and export commands
-#[derive(Debug, Serialize)]
-#[cfg_attr(test, derive(PartialEq))]
-#[cfg_attr(feature = "schema", derive(schemars::JsonSchema))]
-#[cfg_attr(feature = "schema", schemars(default))]
-pub struct CommandsConfig {
-    /// Wrapping shell to parse and execute commands
-    /// If empty, commands will be parsed with shell-words and run natievly
-    pub shell: Vec<String>,
-    /// Default query command for responses
-    #[serde(default)]
-    pub default_query: MimeMap<String>,
+/// Error creating or loading a config file
+#[derive(Debug, Error)]
+pub enum ConfigError {
+    /// Creating a file on disk
+    #[error("Error creating config file {}", path.display())]
+    Create { path: PathBuf, error: io::Error },
+
+    /// Error parsing/deserializing the YAML
+    #[error(transparent)]
+    Yaml(#[from] YamlError),
 }
 
-impl Default for CommandsConfig {
-    fn default() -> Self {
-        // We use the defaults from docker, because it's well tested and
-        // reasonably intuitive
-        // https://docs.docker.com/reference/dockerfile/#shell
-        let default_shell: &[&str] = if cfg!(windows) {
-            &["cmd", "/S", "/C"]
-        } else {
-            &["/bin/sh", "-c"]
-        };
-
-        Self {
-            shell: default_shell.iter().map(ToString::to_string).collect(),
-            default_query: MimeMap::default(),
-        }
-    }
-}
+/// Error opening a configured editor/pager
+#[derive(Debug, Error)]
+#[error("Error opening editor; see {}", doc_link("user_guide/tui/editor"))]
+pub struct EditorError(#[source] EditorBuilderError);
 
 #[cfg(test)]
 mod tests {
@@ -340,7 +260,7 @@ mod tests {
     use env_lock::EnvGuard;
     use pretty_assertions::assert_eq;
     use rstest::{fixture, rstest};
-    use slumber_util::{TempDir, assert_err, temp_dir};
+    use slumber_util::{TempDir, temp_dir};
     use std::fs;
 
     struct ConfigPath {
@@ -391,7 +311,7 @@ mod tests {
     /// readonly
     #[rstest]
     fn test_load_file_exists_readonly(config_path: ConfigPath) {
-        fs::write(&config_path.path, "debug: true\n").unwrap();
+        fs::write(&config_path.path, "large_body_size: 1000\n").unwrap();
         let mut permissions =
             fs::metadata(&config_path.path).unwrap().permissions();
         permissions.set_readonly(true);
@@ -401,8 +321,13 @@ mod tests {
         assert_eq!(
             config,
             Config {
-                debug: true,
-                ..Config::default()
+                editor: None,
+                http: HttpEngineConfig {
+                    large_body_size: 1000,
+                    ..Default::default()
+                },
+                #[cfg(feature = "tui")]
+                tui: TuiConfig::default(),
             }
         );
     }
@@ -447,9 +372,13 @@ mod tests {
 
     /// Loading a config file with contents that don't deserialize correctly
     /// returns an error
+    #[cfg(feature = "tui")] // Extra fields only error if the TUI flag is on
     #[rstest]
     fn test_load_file_invalid(config_path: ConfigPath) {
         fs::write(&config_path.path, "fake_field: true\n").unwrap();
-        assert_err!(Config::load(), "Unexpected field `fake_field`");
+        slumber_util::assert_err(
+            Config::load(),
+            "Unexpected field `fake_field`",
+        );
     }
 }
