@@ -1,10 +1,10 @@
 //! Recipe/folder tree structure
 
 use crate::collection::{Folder, HasId, Recipe, RecipeId};
-use anyhow::anyhow;
 use derive_more::From;
 use indexmap::{IndexMap, map::Values};
 use serde::Serialize;
+use slumber_util::yaml::SourceLocation;
 use strum::EnumDiscriminants;
 use thiserror::Error;
 
@@ -30,61 +30,6 @@ pub struct RecipeTree {
     nodes_by_id: IndexMap<RecipeId, RecipeLookupKey>,
 }
 
-/// A path into the recipe tree. Every constructed path is assumed to be valid,
-/// which must be enforced by the creator.
-#[derive(Clone, Debug, From, Eq, Hash, PartialEq)]
-pub struct RecipeLookupKey(Vec<RecipeId>);
-
-impl RecipeLookupKey {
-    /// How many nodes are above us in the tree?
-    pub fn depth(&self) -> usize {
-        self.0.len() - 1
-    }
-
-    /// Get all parent IDs, starting at the root
-    pub fn ancestors(&self) -> &[RecipeId] {
-        &self.0[0..self.0.len() - 1]
-    }
-}
-
-impl From<&Vec<&RecipeId>> for RecipeLookupKey {
-    fn from(value: &Vec<&RecipeId>) -> Self {
-        Self(value.iter().copied().cloned().collect())
-    }
-}
-
-impl IntoIterator for RecipeLookupKey {
-    type Item = RecipeId;
-    type IntoIter = <Vec<RecipeId> as IntoIterator>::IntoIter;
-
-    fn into_iter(self) -> Self::IntoIter {
-        self.0.into_iter()
-    }
-}
-
-/// A node in the recipe tree, either a folder or recipe
-#[derive(Debug, From, Serialize, EnumDiscriminants)]
-#[strum_discriminants(name(RecipeNodeType))]
-#[cfg_attr(any(test, feature = "test"), derive(PartialEq))]
-#[serde(untagged)]
-#[cfg_attr(feature = "schema", derive(schemars::JsonSchema))]
-pub enum RecipeNode {
-    Folder(Folder),
-    // Rename this variant to match the `requests` field in the root and
-    // folders
-    #[serde(rename = "request")]
-    Recipe(Recipe),
-}
-
-/// Error returned when attempting to build a [RecipeTree] with a duplicate
-/// recipe ID. IDs are unique throughout the entire tree.
-#[derive(Debug, Error)]
-#[error(
-    "Duplicate recipe/folder ID `{0}`; \
-    recipe/folder IDs must be globally unique"
-)]
-pub struct DuplicateRecipeIdError(RecipeId);
-
 impl RecipeTree {
     /// Create a new tree. If there are *any* duplicate IDs in the tree, the
     /// duplicate ID will be returned as an `Err`.
@@ -138,9 +83,11 @@ impl RecipeTree {
     }
 
     /// Get a folder/recipe by ID. Return an error if the ID isn't in the tree
-    pub fn try_get(&self, id: &RecipeId) -> anyhow::Result<&RecipeNode> {
-        self.get(id)
-            .ok_or_else(|| anyhow!("No recipe node with ID `{}`", id,))
+    pub fn try_get(
+        &self,
+        id: &RecipeId,
+    ) -> Result<&RecipeNode, UnknownRecipeError> {
+        self.get(id).ok_or_else(|| UnknownRecipeError(id.clone()))
     }
 
     /// Get a **recipe** by ID. If the ID isn't in the tree, or points to a
@@ -151,9 +98,12 @@ impl RecipeTree {
 
     /// Get a **recipe** by ID. If the ID isn't in the tree, or points to a
     /// folder, return an error that can be presented to the user
-    pub fn try_get_recipe(&self, id: &RecipeId) -> anyhow::Result<&Recipe> {
+    pub fn try_get_recipe(
+        &self,
+        id: &RecipeId,
+    ) -> Result<&Recipe, UnknownRecipeError> {
         self.get_recipe(id)
-            .ok_or_else(|| anyhow!("No recipe with ID `{}`", id,))
+            .ok_or_else(|| UnknownRecipeError(id.clone()))
     }
 
     /// Get all **recipe** IDs in the tree. Useful for printing a list to the
@@ -230,6 +180,52 @@ impl From<IndexMap<RecipeId, RecipeNode>> for RecipeTree {
     }
 }
 
+/// A path into the recipe tree. Every constructed path is assumed to be valid,
+/// which must be enforced by the creator.
+#[derive(Clone, Debug, From, Eq, Hash, PartialEq)]
+pub struct RecipeLookupKey(Vec<RecipeId>);
+
+impl RecipeLookupKey {
+    /// How many nodes are above us in the tree?
+    pub fn depth(&self) -> usize {
+        self.0.len() - 1
+    }
+
+    /// Get all parent IDs, starting at the root
+    pub fn ancestors(&self) -> &[RecipeId] {
+        &self.0[0..self.0.len() - 1]
+    }
+}
+
+impl From<&Vec<&RecipeId>> for RecipeLookupKey {
+    fn from(value: &Vec<&RecipeId>) -> Self {
+        Self(value.iter().copied().cloned().collect())
+    }
+}
+
+impl IntoIterator for RecipeLookupKey {
+    type Item = RecipeId;
+    type IntoIter = <Vec<RecipeId> as IntoIterator>::IntoIter;
+
+    fn into_iter(self) -> Self::IntoIter {
+        self.0.into_iter()
+    }
+}
+
+/// A node in the recipe tree, either a folder or recipe
+#[derive(Debug, From, Serialize, EnumDiscriminants)]
+#[strum_discriminants(name(RecipeNodeType))]
+#[cfg_attr(any(test, feature = "test"), derive(PartialEq))]
+#[serde(untagged)]
+#[cfg_attr(feature = "schema", derive(schemars::JsonSchema))]
+pub enum RecipeNode {
+    Folder(Folder),
+    // Rename this variant to match the `requests` field in the root and
+    // folders
+    #[serde(rename = "request")]
+    Recipe(Recipe),
+}
+
 impl RecipeNode {
     pub fn name(&self) -> &str {
         match self {
@@ -238,6 +234,14 @@ impl RecipeNode {
         }
     }
 
+    /// Get the location where this node is defined in YAML
+    pub fn location(&self) -> &SourceLocation {
+        match self {
+            Self::Folder(folder) => &folder.location,
+            Self::Recipe(recipe) => &recipe.location,
+        }
+    }
+
     /// If this node is a recipe, return it. Otherwise return `None`
     pub fn recipe(&self) -> Option<&Recipe> {
         match self {
@@ -255,6 +259,21 @@ impl RecipeNode {
     }
 }
 
+/// Error returned when attempting to build a [RecipeTree] with a duplicate
+/// recipe ID. IDs are unique throughout the entire tree.
+#[derive(Debug, Error)]
+#[error(
+    "Duplicate recipe/folder ID `{0}`; \
+    recipe/folder IDs must be globally unique"
+)]
+pub struct DuplicateRecipeIdError(RecipeId);
+
+/// Error when requesting a recipe/recipe node by ID that doesn't existing in
+/// the tree
+#[derive(Debug, Error)]
+#[error("No recipe with ID `{0}`")]
+pub struct UnknownRecipeError(pub RecipeId);
+
 #[cfg(test)]
 mod tests {
     use super::*;
