@@ -11,26 +11,28 @@ mod util;
 use crate::collection::Recipe;
 use crate::{
     collection::{Collection, Profile, ProfileId, RecipeId},
-    http::{Exchange, RequestSeed, ResponseRecord, TriggeredRequestError},
+    http::{
+        Exchange, RequestSeed, ResponseRecord, StoredRequestError,
+        TriggeredRequestError,
+    },
     render::{
         functions::RequestTrigger,
         util::{FieldCache, FieldCacheOutcome},
     },
 };
-use anyhow::anyhow;
 use async_trait::async_trait;
 use chrono::Utc;
-use derive_more::{Deref, From};
+use derive_more::{Deref, From, derive::Display};
 use indexmap::IndexMap;
 use itertools::Itertools;
-use serde_json_path::JsonPath;
-use slumber_template::{Arguments, Identifier, LazyValue, RenderError};
-use slumber_util::ResultTraced;
+use serde::Deserialize;
+use slumber_template::{Arguments, Identifier, LazyValue, RenderError, Value};
 use std::{
     fmt::Debug, io, iter, path::PathBuf, process::ExitStatus, sync::Arc,
 };
 use thiserror::Error;
 use tokio::sync::oneshot;
+use tracing::error;
 
 /// A little container struct for all the data that the user can access via
 /// templating. Unfortunately this has to own all data so templating can be
@@ -139,7 +141,7 @@ impl TemplateContext {
         self.http_provider
             .get_latest_request(self.selected_profile.as_ref(), recipe_id)
             .await
-            .map_err(FunctionError::Database)
+            .map_err(FunctionError::StoredRequest)
     }
 
     /// Send a request for the recipe and return the exchange
@@ -173,10 +175,11 @@ impl TemplateContext {
     }
 }
 
-/// A wrapper for [TemplateContext] that provides the [Context] trait. While
-/// [TemplateContext] is intended to be used for multiple renders within a
-/// render group, this is meant for an individual render. As such, it captures
-/// settings that can vary across different renders in the same group.
+/// A wrapper for [TemplateContext] that provides the
+/// [slumber_template::Context] trait. While [TemplateContext] is intended to be
+/// used for multiple renders within a render group, this is meant for an
+/// individual render. As such, it captures settings that can vary across
+/// different renders in the same group.
 #[derive(Debug, Deref)]
 pub struct SingleRenderContext<'a> {
     #[deref]
@@ -271,6 +274,7 @@ impl slumber_template::Context for SingleRenderContext<'_> {
             "file" => functions::file(arguments),
             "float" => functions::float(arguments),
             "integer" => functions::integer(arguments),
+            "jq" => functions::jq(arguments),
             "json_parse" => functions::json_parse(arguments),
             "jsonpath" => functions::jsonpath(arguments),
             "prompt" => functions::prompt(arguments).await,
@@ -360,7 +364,7 @@ pub trait HttpProvider: Debug + Send + Sync {
         &self,
         profile_id: Option<&ProfileId>,
         recipe_id: &RecipeId,
-    ) -> anyhow::Result<Option<Exchange>>;
+    ) -> Result<Option<Exchange>, StoredRequestError>;
 
     /// Build and send an HTTP request. The implementor may choose whether
     /// triggered chained requests will be sent, and whether the result should
@@ -411,9 +415,21 @@ pub struct Select {
     /// Tell the user what we're asking for
     pub message: String,
     /// List of choices the user can pick from
-    pub options: Vec<String>,
-    /// How the prompter will pass the answer back
-    pub channel: ResponseChannel<String>,
+    pub options: Vec<SelectOption>,
+    /// How the prompter will pass the answer back. The returned value is the
+    /// `value` field from the selected [SelectOption]
+    pub channel: ResponseChannel<Value>,
+}
+
+/// An entry in a `select()` list
+#[derive(Debug, Display, Deserialize)]
+#[display("{label}")]
+pub struct SelectOption {
+    /// Label to display to the user for this option
+    pub label: String,
+    /// Underlying value to return if this option is selected. This will be the
+    /// same as the label if the input was a single string.
+    pub value: Value,
 }
 
 /// Channel used to return a response to a one-time request. This is its own
@@ -426,11 +442,9 @@ impl<T> ResponseChannel<T> {
     pub fn respond(self, response: T) {
         // This error *shouldn't* ever happen, because the templating task
         // stays open until it gets a response
-        let _ = self
-            .0
-            .send(response)
-            .map_err(|_| anyhow!("Response listener dropped"))
-            .traced();
+        if self.0.send(response).is_err() {
+            error!("Response listener dropped");
+        }
     }
 }
 
@@ -467,11 +481,6 @@ pub enum FunctionError {
     #[error("Command must have at least one element")]
     CommandEmpty,
 
-    /// An error occurred accessing the persistence database. This error is
-    /// generated by our code so we don't need any extra context.
-    #[error(transparent)]
-    Database(anyhow::Error),
-
     /// Error opening/reading a file
     #[error("Reading file `{path}`")]
     File {
@@ -484,6 +493,11 @@ pub enum FunctionError {
     #[error(transparent)]
     InvalidUtf8(#[from] std::string::FromUtf8Error),
 
+    /// Error executing a jq error. [jaq_core::Error] doesn't impl `Error` or
+    /// `Send` so we just stringify it
+    #[error("{0}")]
+    Jq(String),
+
     /// Error parsing JSON data
     #[error("Error parsing JSON")]
     JsonParse(
@@ -492,19 +506,16 @@ pub enum FunctionError {
         serde_json::Error,
     ),
 
-    /// JSONPath query returned 0 or 2+ results when we expected 1
+    /// jq/JSONPath query returned no results when it should have
+    #[error("No results from JSON query `{query}`")]
+    JsonQueryNoResults { query: String },
+
+    /// jq/JSONPath query returned 2+ results when we expected 1
     #[error(
-        "Expected exactly one result from JSONPath query `{query}`, \
+        "Expected exactly one result from JSON query `{query}`, \
         but got {actual_count}"
     )]
-    JsonPathExactlyOne {
-        query: JsonPath,
-        actual_count: usize,
-    },
-
-    /// JSONPath query returned no results when it should have
-    #[error("No results from JSONPath query `{query}`")]
-    JsonPathNoResults { query: JsonPath },
+    JsonQueryTooMany { query: String, actual_count: usize },
 
     /// An bubbled-up error from rendering a profile field value
     #[error("Rendering profile field `{field}`")]
@@ -541,6 +552,11 @@ pub enum FunctionError {
     #[error("No reply from select")]
     SelectNoReply,
 
+    /// An error occurred while pulling a previous request for a recipe. This
+    /// error is generated by our code so we don't need any extra context.
+    #[error(transparent)]
+    StoredRequest(StoredRequestError),
+
     /// Something bad happened while triggering a request dependency
     #[error("Triggering upstream recipe `{recipe_id}`")]
     Trigger {
