@@ -2,6 +2,7 @@
 
 use anyhow::{Context, Result, anyhow};
 use itertools::Itertools;
+use quote::ToTokens;
 use std::{
     cell::LazyCell,
     collections::HashMap,
@@ -144,6 +145,21 @@ impl TemplateFunctionMetadata {
 
 impl Display for TemplateFunctionMetadata {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        fn fmt_parameters(parameters: &[ParameterMetadata]) -> String {
+            // If the params get too long, overflow onto multiple lines
+            let one_line = parameters.iter().join(", ").to_string();
+            if one_line.len() <= 60 {
+                one_line
+            } else {
+                format!(
+                    "\n{}",
+                    parameters.iter().format_with("", |param, f| f(
+                        &format_args!("  {param},\n")
+                    ))
+                )
+            }
+        }
+
         write!(
             f,
             "### {name}
@@ -154,7 +170,7 @@ impl Display for TemplateFunctionMetadata {
 
 {doc}",
             name = self.name,
-            parameters = self.parameters.iter().join(", "),
+            parameters = fmt_parameters(&self.parameters),
             return_type = self.return_type,
             doc = self.doc_comment,
         )
@@ -234,6 +250,8 @@ enum TypeDef {
     Value,
     /// A homogenous list
     List(&'static Self),
+    /// An object with a static set of fields
+    Struct(&'static [(&'static str, &'static Self)]),
     /// Any of a set of types
     Union(&'static [&'static Self]),
     /// A custom dynamic type. Typically a subset of `String`, such as
@@ -257,9 +275,13 @@ impl TypeDef {
             ty
         };
 
-        TYPE_MAP
-            .with(|map| map.get(ty).copied())
-            .ok_or_else(|| anyhow!("Unmapped type {ty:#?}"))
+        TYPE_MAP.with(|map| map.get(ty).copied()).ok_or_else(|| {
+            anyhow!(
+                "Unmapped type {}; add the type to type_map() in {}",
+                ty.to_token_stream(),
+                file!()
+            )
+        })
     }
 }
 
@@ -273,7 +295,24 @@ impl Display for TypeDef {
             Self::String => write!(f, "string"),
             Self::Literal(literal) => write!(f, "\"{literal}\""),
             Self::Value => write!(f, "value"),
-            Self::List(inner) => write!(f, "{inner}[]"),
+            Self::List(inner) => {
+                // Wrap unions in () to make the grouping clear
+                if let Self::Union(_) = inner {
+                    write!(f, "({inner})[]")
+                } else {
+                    write!(f, "{inner}[]")
+                }
+            }
+            Self::Struct(fields) => {
+                // { "field1": value1, "field2": value2 }
+                write!(
+                    f,
+                    "{{ {} }}",
+                    fields.iter().format_with(", ", |(field, value), f| f(
+                        &format_args!("\"{field}\": {value}")
+                    ))
+                )
+            }
             Self::Union(members) => {
                 write!(f, "{}", members.iter().format(" | "))
             }
@@ -303,10 +342,11 @@ fn type_map() -> HashMap<Type, TypeDef> {
             parse_quote!(CommandOutputMode),
             union!("stdout" | "stderr" | "both"),
         ),
-        (parse_quote!(JsonPath), TypeDef::Custom("JsonPath")),
-        (parse_quote!(JsonPathValue), TypeDef::Value),
+        (parse_quote!(JaqQuery), TypeDef::String),
+        (parse_quote!(JsonPath), TypeDef::String),
+        (parse_quote!(JsonValue), TypeDef::Value),
         (
-            parse_quote!(JsonPathMode),
+            parse_quote!(JsonQueryMode),
             union!("auto" | "single" | "array"),
         ),
         (parse_quote!(RecipeId), TypeDef::String),
@@ -319,6 +359,16 @@ fn type_map() -> HashMap<Type, TypeDef> {
                 &TypeDef::Custom("Duration"),
             ]),
         ),
+        (
+            parse_quote!(Vec<SelectOption>),
+            TypeDef::List(&TypeDef::Union(&[
+                &TypeDef::String,
+                &TypeDef::Struct(&[
+                    ("label", &TypeDef::String),
+                    ("value", &TypeDef::Value),
+                ]),
+            ])),
+        ),
         (parse_quote!(String), TypeDef::String),
         // We're hiding streams from the type system, since they will
         // transparently convert to bytes
