@@ -99,15 +99,6 @@ impl RenderError {
     }
 }
 
-impl de::Error for RenderError {
-    fn custom<T>(msg: T) -> Self
-    where
-        T: Display,
-    {
-        RenderError::Other(msg.to_string().into())
-    }
-}
-
 /// Information about where an error occurred
 #[derive(Debug, Display)]
 pub enum RenderErrorContext {
@@ -157,7 +148,7 @@ pub struct WithValue<E> {
     pub value: Value,
     /// The error that occurred during conversion. This error is transparent,
     /// meaning we include its message in our own `Display` impl and
-    /// [StdError::source] returns its source
+    /// `Error::source` returns its source
     pub error: E,
 }
 
@@ -177,7 +168,7 @@ impl<E> WithValue<E> {
 }
 
 /// An error that can occur while converting from [Value] to some other type.
-/// This is returned from [TryFromValue].
+/// This is returned from [TryFromValue](crate::TryFromValue).
 #[derive(Debug, Error)]
 pub enum ValueError {
     /// Failed to parse a string to a float
@@ -221,6 +212,15 @@ impl ValueError {
     }
 }
 
+impl de::Error for ValueError {
+    fn custom<T>(msg: T) -> Self
+    where
+        T: Display,
+    {
+        Self::Other(msg.to_string().into())
+    }
+}
+
 /// When a value of a particular type is expected but something else is given
 #[derive(Debug, derive_more::Display)]
 pub enum Expected {
