@@ -24,7 +24,7 @@ type RenderResult = Result<LazyValue, RenderError>;
 pub enum Expression {
     /// A literal value such as `3`, `false`, or `"hello"`
     Literal(Literal),
-    /// Field access, as defined by [TemplateContext::get]
+    /// Field access, as defined by [Context::get_field]
     Field(Identifier),
     /// Array literal: `[1, "hello", f()]`
     Array(Vec<Self>),
@@ -175,6 +175,7 @@ impl From<&'static [u8]> for Expression {
     }
 }
 
+// Bytes from byte literal. b"" literals are type &[u8; N], not &[u8]
 impl<const N: usize> From<&'static [u8; N]> for Expression {
     fn from(value: &'static [u8; N]) -> Self {
         value.as_slice().into()
@@ -187,6 +188,13 @@ impl From<Vec<Expression>> for Expression {
     }
 }
 
+/// Object from (key, value) pairs
+impl<const N: usize> From<[(&str, Expression); N]> for Expression {
+    fn from(value: [(&str, Expression); N]) -> Self {
+        Self::Object(value.into_iter().map(|(k, v)| (k.into(), v)).collect())
+    }
+}
+
 impl FromIterator<Expression> for Expression {
     fn from_iter<T: IntoIterator<Item = Self>>(iter: T) -> Self {
         Self::Array(Vec::from_iter(iter))
