@@ -19,7 +19,6 @@ use ratatui::{
     text::{Line, Span},
 };
 use slumber_config::Action;
-use std::fmt::Display;
 
 /// Modal to list and trigger arbitrary actions. The user opens the action menu
 /// with a keybinding, at which point the list of available actions is built
@@ -144,23 +143,39 @@ pub struct MenuAction {
 }
 
 impl MenuAction {
-    /// Get a mapping function to generate menu actions from some type. Useful
-    /// for mapping an iterator of specific action types to this type.
-    pub fn with_data<Data, T>(
-        data: &Data,
+    /// Create a new menu action. This uses the builder-lite pattern to
+    /// customize the created event
+    pub fn new<T: LocalEvent>(
         emitter: Emitter<T>,
-    ) -> impl '_ + Fn(T) -> Self
-    where
-        T: IntoMenuAction<Data>,
-    {
-        move |action| Self {
-            name: action.to_string(),
+        action: T,
+        name: impl Into<String>,
+    ) -> Self {
+        Self {
+            name: name.into(),
             emitter: emitter.upcast(),
-            enabled: action.enabled(data),
-            shortcut: action.shortcut(data),
+            enabled: true,
+            shortcut: None,
             value: Box::new(action),
         }
     }
+
+    /// Enable/disable this action
+    pub fn enable(mut self, enabled: bool) -> Self {
+        self.enabled = enabled;
+        self
+    }
+
+    /// Set/clear the shortcut for this action
+    pub fn shortcut(mut self, shortcut: Option<Action>) -> Self {
+        self.shortcut = shortcut;
+        self
+    }
+
+    /// Is this action enabled?
+    #[cfg(test)]
+    pub fn enabled(&self) -> bool {
+        self.enabled
+    }
 }
 
 impl Generate for &MenuAction {
@@ -185,21 +200,6 @@ impl Generate for &MenuAction {
     }
 }
 
-/// Trait for any type that can be converted into menu actions. This is useful
-/// both for static lists of actions (i.e. enums) and dynamic lists. Combine
-/// with [MenuAction::with_data] to implement [EventHandler::menu_actions].
-pub trait IntoMenuAction<Data>: Display + LocalEvent {
-    /// Should this action be enabled in the menu?
-    fn enabled(&self, _: &Data) -> bool {
-        true
-    }
-
-    /// What input action, if any, should trigger this menu action?
-    fn shortcut(&self, _: &Data) -> Option<Action> {
-        None
-    }
-}
-
 #[cfg(test)]
 mod tests {
     use super::*;
@@ -208,7 +208,6 @@ mod tests {
         view::test_util::TestComponent,
     };
     use rstest::rstest;
-    use strum::{EnumIter, IntoEnumIterator};
     use terminput::KeyCode;
 
     /// A component that provides some actions
@@ -219,9 +218,17 @@ mod tests {
 
     impl EventHandler for Actionable {
         fn menu_actions(&self) -> Vec<MenuAction> {
-            TestMenuAction::iter()
-                .map(MenuAction::with_data(&(), self.emitter))
-                .collect()
+            let emitter = self.emitter;
+            vec![
+                emitter.menu(TestMenuAction::Flobrigate, "Flobrigate"),
+                emitter.menu(TestMenuAction::Profilate, "Profilate"),
+                emitter
+                    .menu(TestMenuAction::Disablify, "Disablify")
+                    .enable(false),
+                emitter
+                    .menu(TestMenuAction::Shortcutticated, "Shortcutticated")
+                    .shortcut(Some(Action::Edit)),
+            ]
         }
     }
 
@@ -235,27 +242,15 @@ mod tests {
         }
     }
 
-    #[derive(Debug, derive_more::Display, PartialEq, EnumIter)]
+    #[derive(Debug, PartialEq)]
     enum TestMenuAction {
+        // Disablify is first to test that disabled actions are skipped
+        Disablify,
         Flobrigate,
         Profilate,
-        Disablify,
         Shortcutticated,
     }
 
-    impl IntoMenuAction<()> for TestMenuAction {
-        fn enabled(&self, &(): &()) -> bool {
-            !matches!(self, Self::Disablify)
-        }
-
-        fn shortcut(&self, &(): &()) -> Option<Action> {
-            match self {
-                Self::Shortcutticated => Some(Action::Edit),
-                _ => None,
-            }
-        }
-    }
-
     /// Test basic action menu interactions
     #[rstest]
     fn test_actions(harness: TestHarness, terminal: TestTerminal) {
@@ -268,9 +263,6 @@ mod tests {
             .action("Profilate")
             .assert_emitted([TestMenuAction::Profilate]);
 
-        // Selecting a disabled action does nothing
-        component.int().action("Disablify").assert_emitted([]);
-
         // Actions can be selected by shortcut
         component
             .int()
