@@ -3,12 +3,7 @@ use crate::{
     http::{RequestMetadata, ResponseMetadata},
     view::{
         RequestState,
-        common::{
-            Pane,
-            actions::{IntoMenuAction, MenuAction},
-            modal::Modal,
-            tabs::Tabs,
-        },
+        common::{Pane, actions::MenuAction, modal::Modal, tabs::Tabs},
         component::{
             Component,
             misc::DeleteRequestModal,
@@ -32,8 +27,8 @@ use ratatui::{
 use serde::{Deserialize, Serialize};
 use slumber_config::Action;
 use slumber_core::collection::RecipeNodeType;
-use std::sync::Arc;
-use strum::{EnumCount, EnumIter, IntoEnumIterator};
+use std::{error::Error, sync::Arc};
+use strum::{EnumCount, EnumIter};
 
 /// Display for a request/response exchange. This allows the user to switch
 /// between request and response. This is bound to a particular [RequestState],
@@ -263,7 +258,7 @@ impl ExchangePaneContent {
             RequestState::Building { .. } => ExchangePaneContentState::Building,
             RequestState::BuildError { error } => {
                 ExchangePaneContentState::BuildError {
-                    error: error.generate(),
+                    error: (error as &dyn Error).generate(),
                 }
             }
             RequestState::Loading { request, .. } => {
@@ -293,7 +288,7 @@ impl ExchangePaneContent {
                 ExchangePaneContentState::RequestError {
                     request: RequestView::new(Arc::clone(&error.request))
                         .into(),
-                    error: error.generate(),
+                    error: (error as &dyn Error).generate(),
                 }
             }
         };
@@ -365,9 +360,63 @@ impl EventHandler for ExchangePaneContent {
     }
 
     fn menu_actions(&self) -> Vec<MenuAction> {
-        ExchangePaneMenuAction::iter()
-            .map(MenuAction::with_data(self, self.actions_emitter))
-            .collect()
+        let emitter = self.actions_emitter;
+        let request = self.state.request();
+        let has_request = request.is_some();
+        let has_request_body = request.is_some_and(RequestView::has_body);
+        let has_response_body = match self.state {
+            ExchangePaneContentState::Building
+            | ExchangePaneContentState::BuildError { .. }
+            | ExchangePaneContentState::Cancelled
+            | ExchangePaneContentState::Loading { .. }
+            | ExchangePaneContentState::RequestError { .. } => false,
+            ExchangePaneContentState::Response { .. } => true,
+        };
+        let selected_tab = self.tabs.data().selected();
+
+        vec![
+            emitter
+                .menu(ExchangePaneMenuAction::CopyUrl, "Copy URL")
+                .enable(has_request),
+            emitter
+                .menu(
+                    ExchangePaneMenuAction::CopyRequestBody,
+                    "Copy Request Body",
+                )
+                .enable(has_request_body),
+            emitter
+                .menu(
+                    ExchangePaneMenuAction::ViewRequestBody,
+                    "View Request Body",
+                )
+                .enable(has_request_body)
+                .shortcut(
+                    (selected_tab == Tab::Request).then_some(Action::View),
+                ),
+            emitter
+                .menu(
+                    ExchangePaneMenuAction::CopyResponseBody,
+                    "Copy Response Body",
+                )
+                .enable(has_response_body),
+            emitter
+                .menu(
+                    ExchangePaneMenuAction::ViewResponseBody,
+                    "View Response Body",
+                )
+                .enable(has_response_body)
+                .shortcut((selected_tab == Tab::Body).then_some(Action::View)),
+            emitter
+                .menu(
+                    ExchangePaneMenuAction::SaveResponseBody,
+                    "Save Response Body as File",
+                )
+                .enable(has_response_body),
+            emitter
+                .menu(ExchangePaneMenuAction::DeleteRequest, "Delete Request")
+                .enable(has_request)
+                .shortcut(Some(Action::Delete)),
+        ]
     }
 
     fn children(&mut self) -> Vec<Component<Child<'_>>> {
@@ -480,11 +529,6 @@ impl ExchangePaneContentState {
         }
     }
 
-    fn has_request_body(&self) -> bool {
-        self.request()
-            .is_some_and(super::request_view::RequestView::has_body)
-    }
-
     fn response(&self) -> Option<&ResponseBodyView> {
         match self {
             Self::Building
@@ -495,74 +539,16 @@ impl ExchangePaneContentState {
             Self::Response { response_body, .. } => Some(response_body.data()),
         }
     }
-
-    fn has_response_body(&self) -> bool {
-        match self {
-            Self::Building
-            | Self::BuildError { .. }
-            | Self::Cancelled
-            | Self::Loading { .. }
-            | Self::RequestError { .. } => false,
-            Self::Response { .. } => true,
-        }
-    }
 }
 
 /// Items in the actions popup menu for the Body
-#[derive(Copy, Clone, Debug, Display, EnumIter)]
+#[derive(Copy, Clone, Debug)]
 enum ExchangePaneMenuAction {
-    #[display("Copy URL")]
     CopyUrl,
-    #[display("Copy Request Body")]
     CopyRequestBody,
-    #[display("View Request Body")]
     ViewRequestBody,
-    #[display("Copy Response Body")]
     CopyResponseBody,
-    #[display("View Response Body")]
     ViewResponseBody,
-    #[display("Save Response Body as File")]
     SaveResponseBody,
-    #[display("Delete Request")]
     DeleteRequest,
 }
-
-impl IntoMenuAction<ExchangePaneContent> for ExchangePaneMenuAction {
-    fn enabled(&self, data: &ExchangePaneContent) -> bool {
-        match self {
-            Self::CopyUrl | Self::DeleteRequest => {
-                data.state.request().is_some()
-            }
-            Self::CopyRequestBody | Self::ViewRequestBody => {
-                data.state.has_request_body()
-            }
-            Self::ViewResponseBody
-            | Self::CopyResponseBody
-            | Self::SaveResponseBody => data.state.has_response_body(),
-        }
-    }
-
-    fn shortcut(&self, data: &ExchangePaneContent) -> Option<Action> {
-        match self {
-            Self::CopyUrl
-            | Self::CopyRequestBody
-            | Self::CopyResponseBody
-            | Self::SaveResponseBody => None,
-            Self::ViewRequestBody => {
-                if matches!(data.tabs.data().selected(), Tab::Request) {
-                    Some(Action::View)
-                } else {
-                    None
-                }
-            }
-            Self::ViewResponseBody => {
-                if matches!(data.tabs.data().selected(), Tab::Body) {
-                    Some(Action::View)
-                } else {
-                    None
-                }
-            }
-            Self::DeleteRequest => Some(Action::Delete),
-        }
-    }
-}
