@@ -6,10 +6,7 @@ use crate::{
     util::ResultReported,
     view::{
         Component, ViewContext,
-        common::{
-            actions::{IntoMenuAction, MenuAction},
-            modal::Modal,
-        },
+        common::{actions::MenuAction, modal::Modal},
         component::{
             collection_select::CollectionSelect,
             exchange_pane::{ExchangePane, ExchangePaneEvent},
@@ -41,10 +38,10 @@ use ratatui::{
 use serde::{Deserialize, Serialize};
 use slumber_config::Action;
 use slumber_core::{
-    collection::{Collection, ProfileId, RecipeId, RecipeNodeType},
+    collection::{Collection, ProfileId, RecipeId, RecipeNode, RecipeNodeType},
     http::RequestId,
 };
-use strum::{EnumCount, EnumIter, IntoEnumIterator};
+use strum::{EnumCount, EnumIter};
 
 /// Primary TUI view, which shows request/response panes
 #[derive(Debug)]
@@ -65,7 +62,7 @@ pub struct PrimaryView {
         Component<ExchangePane>,
     >,
 
-    global_actions_emitter: Emitter<GlobalMenuAction>,
+    global_actions_emitter: Emitter<PrimaryMenuAction>,
 }
 
 impl PrimaryView {
@@ -94,16 +91,13 @@ impl PrimaryView {
     /// Which recipe in the recipe list is selected? `None` iff the list is
     /// empty OR a folder is selected.
     pub fn selected_recipe_id(&self) -> Option<&RecipeId> {
-        self.recipe_list_pane
-            .data()
-            .selected_node()
-            .and_then(|(id, kind)| {
-                if matches!(kind, RecipeNodeType::Recipe) {
-                    Some(id)
-                } else {
-                    None
-                }
-            })
+        self.selected_recipe_node().and_then(|(id, kind)| {
+            if matches!(kind, RecipeNodeType::Recipe) {
+                Some(id)
+            } else {
+                None
+            }
+        })
     }
 
     /// ID of the selected profile. `None` iff the list is empty
@@ -111,6 +105,10 @@ impl PrimaryView {
         self.profile_pane.data().selected_profile_id()
     }
 
+    fn selected_recipe_node(&self) -> Option<(&RecipeId, RecipeNodeType)> {
+        self.recipe_list_pane.data().selected_node()
+    }
+
     /// Get a definition of the request that should be sent from the current
     /// recipe settings
     pub fn request_config(&self) -> Option<RequestConfig> {
@@ -235,6 +233,19 @@ impl PrimaryView {
             }
         }
     }
+
+    /// Send a message to open the collection file to the selected
+    /// recipe/folder. If there are no recipes, just open to the start
+    fn edit_collection(&self) {
+        let collection = ViewContext::collection();
+        // Get the source location of the selected folder/recipe
+        let location = self
+            .selected_recipe_node()
+            .and_then(|(id, _)| collection.recipes.get(id))
+            .map(RecipeNode::location)
+            .cloned();
+        ViewContext::send_message(Message::CollectionEdit { location });
+    }
 }
 
 impl EventHandler for PrimaryView {
@@ -317,20 +328,27 @@ impl EventHandler for PrimaryView {
                         .select(&PrimaryPane::Exchange),
                 }
             })
+            // Handle our own menu action type
             .emitted(self.global_actions_emitter, |menu_action| {
-                // Handle our own menu action type
                 match menu_action {
-                    GlobalMenuAction::EditCollection => {
-                        ViewContext::send_message(Message::CollectionEdit);
+                    PrimaryMenuAction::EditCollection => {
+                        self.edit_collection();
                     }
                 }
             })
     }
 
     fn menu_actions(&self) -> Vec<MenuAction> {
-        GlobalMenuAction::iter()
-            .map(MenuAction::with_data(self, self.global_actions_emitter))
-            .collect()
+        let emitter = self.global_actions_emitter;
+        let edit_collection_name = match self.selected_recipe_node() {
+            None => "Edit Collection",
+            Some((_, RecipeNodeType::Folder)) => "Edit Folder",
+            Some((_, RecipeNodeType::Recipe)) => "Edit Recipe",
+        };
+        vec![
+            emitter
+                .menu(PrimaryMenuAction::EditCollection, edit_collection_name),
+        ]
     }
 
     fn children(&mut self) -> Vec<Component<Child<'_>>> {
@@ -369,11 +387,8 @@ impl<'a> Draw<PrimaryViewProps<'a>> for PrimaryView {
         );
 
         let collection = ViewContext::collection();
-        let selected_recipe_node = self
-            .recipe_list_pane
-            .data()
-            .selected_node()
-            .and_then(|(id, _)| {
+        let selected_recipe_node =
+            self.selected_recipe_node().and_then(|(id, _)| {
                 collection
                     .recipes
                     .try_get(id)
@@ -398,10 +413,7 @@ impl<'a> Draw<PrimaryViewProps<'a>> for PrimaryView {
             || {
                 ExchangePane::new(
                     props.selected_request,
-                    self.recipe_list_pane
-                        .data()
-                        .selected_node()
-                        .map(|(_, node_type)| node_type),
+                    self.selected_recipe_node().map(|(_, node_type)| node_type),
                 )
                 .into()
             },
@@ -461,15 +473,13 @@ enum FullscreenMode {
 }
 
 /// Menu actions available in all contexts
-#[derive(Copy, Clone, Debug, Display, EnumIter)]
-enum GlobalMenuAction {
-    /// Open the current collection file in an external editor
-    #[display("Edit Collection")]
+#[derive(Copy, Clone, Debug)]
+enum PrimaryMenuAction {
+    /// Open the collection file in an external editor, jumping to whatever
+    /// recipe/folder is currently selected
     EditCollection,
 }
 
-impl IntoMenuAction<PrimaryView> for GlobalMenuAction {}
-
 /// Helper for adjusting pane behavior according to state
 struct Panes {
     profile: PaneState,
@@ -553,17 +563,22 @@ mod tests {
         assert_eq!(component.data().request_config(), Some(expected_config));
     }
 
-    /// Test "Edit Collection" action
+    /// Test "Edit Recipe" action
     #[rstest]
-    fn test_edit_collection(mut harness: TestHarness, terminal: TestTerminal) {
+    fn test_edit_recipe(mut harness: TestHarness, terminal: TestTerminal) {
         let mut component = create_component(&mut harness, &terminal);
         component.int().drain_draw().assert_empty();
 
         harness.clear_messages(); // Clear init junk
 
-        component.int().action("Edit Collection").assert_empty();
+        component.int().action("Edit Recipe").assert_empty();
         // Event should be converted into a message appropriately
-        assert_matches!(harness.pop_message_now(), Message::CollectionEdit);
+        assert_matches!(
+            harness.pop_message_now(),
+            // The actual location is unimportant because the collection was
+            // generated in memory, but make sure it's populated
+            Message::CollectionEdit { location: Some(_) }
+        );
     }
 
     /// Test "Copy URL" action, which is available via the Recipe List or Recipe
