@@ -1,24 +1,17 @@
-use crate::{
-    util::ResultReported,
-    view::{
-        ViewContext,
-        common::{
-            actions::{IntoMenuAction, MenuAction},
-            modal::Modal,
-            table::{Table, ToggleRow},
-            text_box::TextBox,
-        },
-        component::{
-            Component,
-            misc::TextBoxModal,
-            recipe_pane::persistence::{RecipeOverrideKey, RecipeTemplate},
-        },
-        context::UpdateContext,
-        draw::{Draw, DrawMetadata, Generate},
-        event::{Child, Emitter, Event, EventHandler, OptionEvent, ToEmitter},
-        state::select::{SelectState, SelectStateEvent, SelectStateEventType},
-        util::persistence::{Persisted, PersistedKey, PersistedLazy},
+use crate::view::{
+    common::{
+        actions::MenuAction,
+        table::{Table, ToggleRow},
     },
+    component::{
+        Component,
+        recipe_pane::persistence::{RecipeOverrideKey, RecipeTemplate},
+    },
+    context::UpdateContext,
+    draw::{Draw, DrawMetadata, Generate},
+    event::{Child, Emitter, Event, EventHandler, OptionEvent, ToEmitter},
+    state::select::{SelectState, SelectStateEvent, SelectStateEventType},
+    util::persistence::{Persisted, PersistedKey, PersistedLazy},
 };
 use itertools::Itertools;
 use ratatui::{
@@ -148,7 +141,10 @@ where
             })
             .emitted(
                 self.override_emitter,
-                |SaveRecipeTableOverride { row_index, value }| {
+                |SaveRecipeTableOverride {
+                     row_index,
+                     template,
+                 }| {
                     // The row we're modifying *should* still be the selected
                     // row, because it shouldn't be possible to change the
                     // selection while the edit modal is open. It's safer to
@@ -156,27 +152,34 @@ where
                     // got the right one.
                     self.select.data_mut().get_mut().items_mut()[row_index]
                         .value
-                        .set_override(&value);
+                        .value
+                        .set_override(template);
                 },
             )
             .emitted(self.actions_emitter, |menu_action| match menu_action {
                 // The selected row can't change while the action menu is open,
                 // so we don't need to plumb the index/key through
-                RecipeTableMenuAction::Edit { .. } => self.edit_selected_row(),
-                RecipeTableMenuAction::Reset { .. } => {
+                RecipeTableMenuAction::Edit => self.edit_selected_row(),
+                RecipeTableMenuAction::Reset => {
                     self.reset_selected_row();
                 }
             })
     }
 
     fn menu_actions(&self) -> Vec<MenuAction> {
-        [
-            RecipeTableMenuAction::Edit { noun: self.noun },
-            RecipeTableMenuAction::Reset { noun: self.noun },
+        let emitter = self.actions_emitter;
+        let noun = self.noun;
+        let selected = self.select.data().selected();
+        vec![
+            emitter
+                .menu(RecipeTableMenuAction::Edit, format!("Edit {noun}"))
+                .enable(selected.is_some())
+                .shortcut(Some(Action::Edit)),
+            emitter
+                .menu(RecipeTableMenuAction::Reset, format!("Reset {noun}"))
+                .enable(selected.is_some_and(|row| row.value.is_overridden()))
+                .shortcut(Some(Action::Reset)),
         ]
-        .into_iter()
-        .map(MenuAction::with_data(self, self.actions_emitter))
-        .collect()
     }
 
     fn children(&mut self) -> Vec<Component<Child<'_>>> {
@@ -221,46 +224,13 @@ where
 #[derive(Debug)]
 struct SaveRecipeTableOverride {
     row_index: usize,
-    value: String,
+    template: Template,
 }
 
-#[derive(Debug, derive_more::Display)]
+#[derive(Debug)]
 enum RecipeTableMenuAction {
-    #[display("Edit {noun}")]
-    Edit { noun: &'static str },
-    #[display("Reset {noun}")]
-    Reset { noun: &'static str },
-}
-
-impl<RowSelectKey, RowToggleKey>
-    IntoMenuAction<RecipeFieldTable<RowSelectKey, RowToggleKey>>
-    for RecipeTableMenuAction
-where
-    RowSelectKey: PersistedKey<Value = Option<String>>,
-    RowToggleKey: PersistedKey<Value = bool>,
-{
-    fn enabled(
-        &self,
-        data: &RecipeFieldTable<RowSelectKey, RowToggleKey>,
-    ) -> bool {
-        let selected = data.select.data().selected();
-        match self {
-            Self::Edit { .. } => selected.is_some(),
-            Self::Reset { .. } => {
-                selected.is_some_and(|row| row.value.is_overridden())
-            }
-        }
-    }
-
-    fn shortcut(
-        &self,
-        _: &RecipeFieldTable<RowSelectKey, RowToggleKey>,
-    ) -> Option<Action> {
-        match self {
-            Self::Edit { .. } => Some(Action::Edit),
-            Self::Reset { .. } => Some(Action::Reset),
-        }
-    }
+    Edit,
+    Reset,
 }
 
 #[derive(Debug)]
@@ -321,33 +291,16 @@ impl<K: PersistedKey<Value = bool>> RowState<K> {
     /// Open a modal to create or edit the value's temporary override
     fn open_edit_modal(&self, emitter: Emitter<SaveRecipeTableOverride>) {
         let index = self.index;
-        TextBoxModal::new(
+        self.value.open_edit_modal(
             format!("Edit value for {}", self.key),
-            TextBox::default()
-                // Edit as a raw template
-                .default_value(self.value.template().display().into_owned())
-                .validator(|value| value.parse::<Template>().is_ok()),
-            move |value| {
+            move |template| {
                 // Defer the state update into an event, so it can get &mut
                 emitter.emit(SaveRecipeTableOverride {
                     row_index: index,
-                    value,
+                    template,
                 });
             },
-        )
-        .open();
-    }
-
-    /// Override the value template and re-render the preview
-    fn set_override(&mut self, override_value: &str) {
-        // The validator on the override text box enforces that it's a valid
-        // template, so we expect this parse to succeed
-        if let Some(template) = override_value
-            .parse::<Template>()
-            .reported(&ViewContext::messages_tx())
-        {
-            self.value.set_override(template);
-        }
+        );
     }
 
     /// Get the disabled/override state of this row
