@@ -1,5 +1,5 @@
 use anyhow::Context;
-use slumber_util::{ResultTraced, paths};
+use slumber_util::{ResultTracedAnyhow, paths};
 use std::{
     fs::{self, File, OpenOptions},
     io,
@@ -41,11 +41,7 @@ async fn main() -> anyhow::Result<std::process::ExitCode> {
             // Do *not* return the error, because that prints a stack trace
             // which is way too verbose. Just print the error messages instead
             .unwrap_or_else(|error| {
-                eprintln!("{error}");
-                error
-                    .chain()
-                    .skip(1)
-                    .for_each(|cause| eprintln!("  {cause}"));
+                slumber_cli::print_error(&error);
                 ExitCode::FAILURE
             })),
     }
